# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/identity_create.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_IdentityCreateDialog(object):
    def setupUi(self, IdentityCreateDialog):
        IdentityCreateDialog.setObjectName("IdentityCreateDialog")
        IdentityCreateDialog.resize(836, 195)
        self.verticalLayout = QtWidgets.QVBoxLayout(IdentityCreateDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.helpLabel = QtWidgets.QLabel(IdentityCreateDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.helpLabel.setFont(font)
        self.helpLabel.setObjectName("helpLabel")
        self.verticalLayout.addWidget(self.helpLabel)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.addressLabel = QtWidgets.QLabel(IdentityCreateDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.addressLabel.sizePolicy().hasHeightForWidth())
        self.addressLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.addressLabel.setObjectName("addressLabel")
        self.gridLayout.addWidget(self.addressLabel, 1, 0, 1, 1)
        self.accountComboBox = QtWidgets.QComboBox(IdentityCreateDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountComboBox.sizePolicy().hasHeightForWidth())
        self.accountComboBox.setSizePolicy(sizePolicy)
        self.accountComboBox.setObjectName("accountComboBox")
        self.gridLayout.addWidget(self.accountComboBox, 0, 1, 1, 1)
        self.addressValueLabel = QtWidgets.QLabel(IdentityCreateDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.addressValueLabel.sizePolicy().hasHeightForWidth())
        self.addressValueLabel.setSizePolicy(sizePolicy)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.gridLayout.addWidget(self.addressValueLabel, 1, 1, 1, 1)
        self.identityIndexValueLabel = QtWidgets.QLabel(IdentityCreateDialog)
        self.identityIndexValueLabel.setStyleSheet("color: blue;")
        self.identityIndexValueLabel.setText("")
        self.identityIndexValueLabel.setObjectName("identityIndexValueLabel")
        self.gridLayout.addWidget(self.identityIndexValueLabel, 2, 1, 1, 1)
        self.accountLabel = QtWidgets.QLabel(IdentityCreateDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountLabel.sizePolicy().hasHeightForWidth())
        self.accountLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.accountLabel.setFont(font)
        self.accountLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.accountLabel.setObjectName("accountLabel")
        self.gridLayout.addWidget(self.accountLabel, 0, 0, 1, 1)
        self.identityIndexLabel = QtWidgets.QLabel(IdentityCreateDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.identityIndexLabel.sizePolicy().hasHeightForWidth())
        self.identityIndexLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.identityIndexLabel.setFont(font)
        self.identityIndexLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.identityIndexLabel.setObjectName("identityIndexLabel")
        self.gridLayout.addWidget(self.identityIndexLabel, 2, 0, 1, 1)
        self.createIdentityButton = QtWidgets.QPushButton(IdentityCreateDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.createIdentityButton.sizePolicy().hasHeightForWidth())
        self.createIdentityButton.setSizePolicy(sizePolicy)
        self.createIdentityButton.setObjectName("createIdentityButton")
        self.gridLayout.addWidget(self.createIdentityButton, 0, 2, 1, 1)
        self.identityStatusLabel = QtWidgets.QLabel(IdentityCreateDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.identityStatusLabel.setFont(font)
        self.identityStatusLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.identityStatusLabel.setObjectName("identityStatusLabel")
        self.gridLayout.addWidget(self.identityStatusLabel, 3, 0, 1, 1)
        self.identityStatusValueLabel = QtWidgets.QLabel(IdentityCreateDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.identityStatusValueLabel.sizePolicy().hasHeightForWidth())
        self.identityStatusValueLabel.setSizePolicy(sizePolicy)
        self.identityStatusValueLabel.setText("")
        self.identityStatusValueLabel.setObjectName("identityStatusValueLabel")
        self.gridLayout.addWidget(self.identityStatusValueLabel, 3, 1, 1, 1)
        self.loaderIconLabel = QtWidgets.QLabel(IdentityCreateDialog)
        self.loaderIconLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.loaderIconLabel.setText("")
        self.loaderIconLabel.setScaledContents(True)
        self.loaderIconLabel.setObjectName("loaderIconLabel")
        self.gridLayout.addWidget(self.loaderIconLabel, 1, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.errorLabel = QtWidgets.QLabel(IdentityCreateDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorLabel.sizePolicy().hasHeightForWidth())
        self.errorLabel.setSizePolicy(sizePolicy)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel)
        self.buttonBox = QtWidgets.QDialogButtonBox(IdentityCreateDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(IdentityCreateDialog)
        self.buttonBox.accepted.connect(IdentityCreateDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(IdentityCreateDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(IdentityCreateDialog)

    def retranslateUi(self, IdentityCreateDialog):
        #  _translate = QtCore.QCoreApplication.translate
        IdentityCreateDialog.setWindowTitle(_("Create Identity"))
        self.helpLabel.setText(_("Select the account of the new identity"))
        self.addressLabel.setText(_("Address"))
        self.accountLabel.setText(_("Account"))
        self.identityIndexLabel.setText(_("Identity Index"))
        self.createIdentityButton.setText(_("Create Identity"))
        self.identityStatusLabel.setText(_("Identity Status"))
import tikka.slots.pyqt.resources.icons.index_rc
