# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/account_certification_row.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountCertificationRowWidget(object):
    def setupUi(self, AccountCertificationRowWidget):
        AccountCertificationRowWidget.setObjectName("AccountCertificationRowWidget")
        AccountCertificationRowWidget.resize(490, 126)
        self.verticalLayout = QtWidgets.QVBoxLayout(AccountCertificationRowWidget)
        self.verticalLayout.setContentsMargins(-1, 0, -1, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout.setVerticalSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.accountAddressValuelabel = QtWidgets.QLabel(AccountCertificationRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountAddressValuelabel.sizePolicy().hasHeightForWidth())
        self.accountAddressValuelabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Courier")
        self.accountAddressValuelabel.setFont(font)
        self.accountAddressValuelabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.accountAddressValuelabel.setObjectName("accountAddressValuelabel")
        self.horizontalLayout_2.addWidget(self.accountAddressValuelabel)
        self.identityNameValueLabel = QtWidgets.QLabel(AccountCertificationRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.identityNameValueLabel.sizePolicy().hasHeightForWidth())
        self.identityNameValueLabel.setSizePolicy(sizePolicy)
        self.identityNameValueLabel.setIndent(10)
        self.identityNameValueLabel.setObjectName("identityNameValueLabel")
        self.horizontalLayout_2.addWidget(self.identityNameValueLabel)
        self.accountNameValueLabel = QtWidgets.QLabel(AccountCertificationRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountNameValueLabel.sizePolicy().hasHeightForWidth())
        self.accountNameValueLabel.setSizePolicy(sizePolicy)
        self.accountNameValueLabel.setObjectName("accountNameValueLabel")
        self.horizontalLayout_2.addWidget(self.accountNameValueLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.identityStatusValueLabel = QtWidgets.QLabel(AccountCertificationRowWidget)
        self.identityStatusValueLabel.setObjectName("identityStatusValueLabel")
        self.verticalLayout_2.addWidget(self.identityStatusValueLabel)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.createdOnValueLabel = QtWidgets.QLabel(AccountCertificationRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.createdOnValueLabel.sizePolicy().hasHeightForWidth())
        self.createdOnValueLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setItalic(True)
        self.createdOnValueLabel.setFont(font)
        self.createdOnValueLabel.setStyleSheet("color: \"gray\"")
        self.createdOnValueLabel.setObjectName("createdOnValueLabel")
        self.horizontalLayout.addWidget(self.createdOnValueLabel)
        self.updatedOnValueLabel = QtWidgets.QLabel(AccountCertificationRowWidget)
        font = QtGui.QFont()
        font.setItalic(True)
        self.updatedOnValueLabel.setFont(font)
        self.updatedOnValueLabel.setStyleSheet("color: \"gray\"")
        self.updatedOnValueLabel.setObjectName("updatedOnValueLabel")
        self.horizontalLayout.addWidget(self.updatedOnValueLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout_2, 0, 1, 1, 1)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_3.setSpacing(6)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.expireOnValueLabel = QtWidgets.QLabel(AccountCertificationRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.expireOnValueLabel.sizePolicy().hasHeightForWidth())
        self.expireOnValueLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.expireOnValueLabel.setFont(font)
        self.expireOnValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid green;\n"
"border-radius: 10%;\n"
"background: green;")
        self.expireOnValueLabel.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.expireOnValueLabel.setObjectName("expireOnValueLabel")
        self.verticalLayout_3.addWidget(self.expireOnValueLabel, 0, QtCore.Qt.AlignBottom)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)
        self.gridLayout.addLayout(self.verticalLayout_3, 0, 4, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.line = QtWidgets.QFrame(AccountCertificationRowWidget)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)

        self.retranslateUi(AccountCertificationRowWidget)
        QtCore.QMetaObject.connectSlotsByName(AccountCertificationRowWidget)

    def retranslateUi(self, AccountCertificationRowWidget):
        #  _translate = QtCore.QCoreApplication.translate
        AccountCertificationRowWidget.setWindowTitle(_("CertificationRowWidget"))
        self.accountAddressValuelabel.setText(_("address"))
        self.identityNameValueLabel.setText(_("name#0000"))
        self.accountNameValueLabel.setText(_(" - Name"))
        self.identityStatusValueLabel.setText(_("status"))
        self.createdOnValueLabel.setText(_("datetime"))
        self.updatedOnValueLabel.setText(_("(datetime)"))
        self.expireOnValueLabel.setText(_("datetime"))
