# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/profile_change_account.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ProfileChangeAccountDialog(object):
    def setupUi(self, ProfileChangeAccountDialog):
        ProfileChangeAccountDialog.setObjectName("ProfileChangeAccountDialog")
        ProfileChangeAccountDialog.resize(738, 172)
        self.verticalLayout = QtWidgets.QVBoxLayout(ProfileChangeAccountDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.helpLabel = QtWidgets.QLabel(ProfileChangeAccountDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.helpLabel.setFont(font)
        self.helpLabel.setObjectName("helpLabel")
        self.verticalLayout.addWidget(self.helpLabel)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.addressLabel = QtWidgets.QLabel(ProfileChangeAccountDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.addressLabel.sizePolicy().hasHeightForWidth())
        self.addressLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.addressLabel.setObjectName("addressLabel")
        self.gridLayout.addWidget(self.addressLabel, 1, 0, 1, 1)
        self.accountLabel = QtWidgets.QLabel(ProfileChangeAccountDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountLabel.sizePolicy().hasHeightForWidth())
        self.accountLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.accountLabel.setFont(font)
        self.accountLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.accountLabel.setObjectName("accountLabel")
        self.gridLayout.addWidget(self.accountLabel, 0, 0, 1, 1)
        self.profileLabel = QtWidgets.QLabel(ProfileChangeAccountDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.profileLabel.setFont(font)
        self.profileLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.profileLabel.setObjectName("profileLabel")
        self.gridLayout.addWidget(self.profileLabel, 2, 0, 1, 1)
        self.profileValueLabel = QtWidgets.QLabel(ProfileChangeAccountDialog)
        self.profileValueLabel.setText("")
        self.profileValueLabel.setObjectName("profileValueLabel")
        self.gridLayout.addWidget(self.profileValueLabel, 2, 1, 1, 1)
        self.accountComboBox = QtWidgets.QComboBox(ProfileChangeAccountDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountComboBox.sizePolicy().hasHeightForWidth())
        self.accountComboBox.setSizePolicy(sizePolicy)
        self.accountComboBox.setObjectName("accountComboBox")
        self.gridLayout.addWidget(self.accountComboBox, 0, 1, 1, 1)
        self.addressValueLabel = QtWidgets.QLabel(ProfileChangeAccountDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.addressValueLabel.sizePolicy().hasHeightForWidth())
        self.addressValueLabel.setSizePolicy(sizePolicy)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.gridLayout.addWidget(self.addressValueLabel, 1, 1, 1, 1)
        self.changeAccountButton = QtWidgets.QPushButton(ProfileChangeAccountDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.changeAccountButton.sizePolicy().hasHeightForWidth())
        self.changeAccountButton.setSizePolicy(sizePolicy)
        self.changeAccountButton.setObjectName("changeAccountButton")
        self.gridLayout.addWidget(self.changeAccountButton, 0, 2, 1, 1)
        self.loaderIconLabel = QtWidgets.QLabel(ProfileChangeAccountDialog)
        self.loaderIconLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.loaderIconLabel.setText("")
        self.loaderIconLabel.setScaledContents(True)
        self.loaderIconLabel.setObjectName("loaderIconLabel")
        self.gridLayout.addWidget(self.loaderIconLabel, 0, 3, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.errorLabel = QtWidgets.QLabel(ProfileChangeAccountDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorLabel.sizePolicy().hasHeightForWidth())
        self.errorLabel.setSizePolicy(sizePolicy)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel)
        self.buttonBox = QtWidgets.QDialogButtonBox(ProfileChangeAccountDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(ProfileChangeAccountDialog)
        self.buttonBox.accepted.connect(ProfileChangeAccountDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(ProfileChangeAccountDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(ProfileChangeAccountDialog)

    def retranslateUi(self, ProfileChangeAccountDialog):
        #  _translate = QtCore.QCoreApplication.translate
        ProfileChangeAccountDialog.setWindowTitle(_("Change account of profile"))
        self.helpLabel.setText(_("Select new account for profile"))
        self.addressLabel.setText(_("Address"))
        self.accountLabel.setText(_("Account"))
        self.profileLabel.setText(_("Profile"))
        self.changeAccountButton.setText(_("Change account"))
import tikka.slots.pyqt.resources.icons.index_rc
